#ifndef _SOCKET_H_
#define	_SOCKET_H_

#include <sys/fcntlcom.h>

/*
 * Level number for (get/set)sockopt() to apply to socket itself.
 */
#define SOL_SOCKET	0xffff		/* options for socket level */

/*
 * Additional options, not kept in so_options.
 */
#define SO_SNDBUF	0x1001		/* send buffer size */
#define SO_RCVBUF	0x1002		/* receive buffer size */
#define SO_SNDLOWAT	0x1003		/* send low-water mark */
#define SO_RCVLOWAT	0x1004		/* receive low-water mark */
#define SO_SNDTIMEO	0x1005		/* send timeout */
#define SO_RCVTIMEO	0x1006		/* receive timeout */
#define SO_ERROR	0x1007		/* get error status and clear */
#define SO_TYPE		0x1008		/* get socket type */

/*
 * Message header for recvmsg and sendmsg calls.
 */
struct msghdr {
	caddr_t	msg_name;		/* optional address */
	int	msg_namelen;		/* size of address */
	struct	iovec *msg_iov;		/* scatter/gather array */
	int	msg_iovlen;		/* # elements in msg_iov */
	caddr_t	msg_accrights;		/* access rights sent/received */
	int	msg_accrightslen;
};


/*
 * Address families.
 */
#define AF_UNSPEC	0		/* unspecified */
#define AF_UNIX		1		/* local to host (pipes, portals) */
#define AF_INET		2		/* internetwork: UDP, TCP, etc. */
#define AF_IMPLINK	3		/* arpanet imp addresses */
#define AF_PUP		4		/* pup protocols: e.g. BSP */
#define AF_CHAOS	5		/* mit CHAOS protocols */
#define AF_NS		6		/* XEROX NS protocols */
#define AF_NBS		7		/* nbs protocols */
#define AF_ECMA		8		/* european computer manufacturers */
#define AF_DATAKIT	9		/* datakit protocols */
#define AF_CCITT	10		/* CCITT protocols, X.25 etc */
#define AF_SNA		11		/* IBM SNA */
#define AF_DECnet	12		/* DECnet */
#define AF_DLI		13		/* Direct data link interface */
#define AF_LAT		14		/* LAT */
#define AF_HYLINK	15		/* NSC Hyperchannel */
#define AF_APPLETALK	16		/* Apple Talk */

#define AF_MAX		17

/*
 * Types
 */
#define SOCK_STREAM	1		/* stream socket */
#define SOCK_DGRAM	2		/* datagram socket */
#define SOCK_RAW	3		/* raw-protocol interface */
#define SOCK_RDM	4		/* reliably-delivered message */
#define SOCK_SEQPACKET	5		/* sequenced packet stream */


/*
 * Structure used by kernel to store most
 * addresses.
 */
struct sockaddr {
	u_short	sa_family;		/* address family */
	char	sa_data[14];		/* up to 14 bytes of direct address */
};

#include <sys/cdefs.h>
#include <sys/__unistd.h>

__BEGIN_DECLS
ssize_t	sendto __P((int, const void *, size_t, int,
		const struct sockaddr *, int));
__END_DECLS

/**********************************************************************/
/**********************************************************************/


#endif
